@@ -10,7 +10,7 @@ from tempfile import mkstemp
 from tests import TestCase, DATA_DIR
 from mutagen.mp4 import (MP4, Atom, Atoms, MP4Tags, MP4Info, delete, MP4Cover,
                          MP4MetadataError, MP4FreeForm, error, AtomDataType,
-                         MP4MetadataValueError, AtomError)
+                         AtomError, _item_sort_key)
 from mutagen.mp4._util import parse_full_atom
 from mutagen.mp4._as_entry import AudioSampleEntry, ASEntryError
 from mutagen._util import cdata
@@ -173,7 +173,7 @@ class TMP4Tags(TestCase):
     def test_parse_multiple_atoms(self):
         # while we don't write multiple values as multiple atoms
         # still read them
-        # https://bitbucket.org/lazka/mutagen/issue/165
+        # https://github.com/quodlibet/mutagen/issues/165
         data = Atom.render(b"data", b"\x00\x00\x00\x01" + b"\x00" * 4 + b"foo")
         grp1 = Atom.render(b"\xa9grp", data)
         data = Atom.render(b"data", b"\x00\x00\x00\x01" + b"\x00" * 4 + b"bar")
@@ -407,11 +407,11 @@ class TMP4Tags(TestCase):
     def test_pprint_non_text_list(self):
         tags = MP4Tags()
         tags["tmpo"] = [120, 121]
-        tags["trck"] = [(1, 2), (3, 4)]
+        tags["trkn"] = [(1, 2), (3, 4)]
         tags.pprint()
 
     def test_freeform_data(self):
-        # http://code.google.com/p/mutagen/issues/detail?id=103
+        # https://github.com/quodlibet/mutagen/issues/103
         key = "----:com.apple.iTunes:Encoding Params"
         value = (b"vers\x00\x00\x00\x01acbf\x00\x00\x00\x01brat\x00\x01\xf4"
                  b"\x00cdcv\x00\x01\x05\x04")
@@ -454,6 +454,9 @@ class TMP4Mixin(object):
             return
         self.assertEqual(call_faad("-w", self.filename), 0)
 
+    def test_set_inval(self):
+        self.assertRaises(TypeError, self.audio.__setitem__, "\xa9nam", 42)
+
     def test_score(self):
         fileobj = open(self.filename, "rb")
         header = fileobj.read(128)
@@ -540,7 +543,7 @@ class TMP4Mixin(object):
         try:
             self.set_key('\xa9nam', [b'\xe3\x82\x8a\xe3\x81\x8b'],
                          result=[u'\u308a\u304b'])
-        except MP4MetadataValueError:
+        except TypeError:
             if not PY3:
                 raise
 
@@ -550,7 +553,7 @@ class TMP4Mixin(object):
 
     def test_invalid_text(self):
         self.assertRaises(
-            MP4MetadataValueError, self.set_key, '\xa9nam', [b'\xff'])
+            TypeError, self.audio.__setitem__, '\xa9nam', [b'\xff'])
 
     def test_save_text(self):
         self.set_key('\xa9nam', [u"Some test name"])
@@ -932,7 +935,7 @@ class TMP4Misc(TestCase):
             ("----", ["1234"]),
         ]
 
-        sorted_items = sorted(items, key=MP4Tags._key_sort)
+        sorted_items = sorted(items, key=lambda kv: _item_sort_key(*kv))
         self.assertEqual(sorted_items, items)
 
 
